package com.huimv.acquisition.socket.eartagrecive;

import com.huimv.acquisition.socket.eartagrecive.service.IClientDataPacketService;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;

public class ServerThread01 extends Thread {


    private IClientDataPacketService iClientDataPacketService;

    //16进制数字字符集
    public static final String HEXSTRING = "0123456789ABCDEF";
    //心跳超时时间
    private static final int TIMEOUT = 60 * 1000;

    private Socket m_socket;
    //接收到数据的最新时间
    private long m_lastReceiveTime = System.currentTimeMillis();
    //该线程是否正在运行
    private boolean m_isRuning = false;

    public ServerThread01(Socket socket , IClientDataPacketService iClientDataPacketService)
    {
        this.m_socket = socket;
        this.iClientDataPacketService = iClientDataPacketService;
    }

    @Override
    public void start()
    {
        if (m_isRuning)
        {
            System.out.println(">>>线程" + this.getId() + "启动失败,该线程正在执行");
            return;
        }
        else
        {
            m_isRuning = true;
            super.start();
        }
    }

    @Override
    public void run()
    {
        //字节输入流
        InputStream inputStream = null;
        //字节输出流
        OutputStream outputStream = null;
        try
        {
            inputStream = m_socket.getInputStream();
            outputStream = m_socket.getOutputStream();
            String info = "";
            //按byte读
            byte[] bytes = new byte[1];
            while (m_isRuning)
            {
                //检测心跳
                if (System.currentTimeMillis() - m_lastReceiveTime > TIMEOUT)
                {
                    m_isRuning = false;
                    //跳出,执行finally块
                    break;
                }
                //返回下次调用可以不受阻塞地从此流读取或跳过的估计字节数,如果等于0则表示已经读完
                if (inputStream.available() > 0)
                {
                    //重置接收到数据的最新时间
                    m_lastReceiveTime = System.currentTimeMillis();

                    byte[] b = new byte[1024];

                    inputStream.read(b);

                    String packet = new String(b);

                    packet = packet.trim();

                    System.out.println(">>>线程" + this.getId() + "收到:" + packet);
                    // 处理数据包
                    String ss = iClientDataPacketService.handleClientSendPacket(packet);
                    //粘包情况需要处理
                    String tempStr = ss ;

                    info += tempStr;
                    //已经读完
                    if (inputStream.available() == 0)
                    {
                        System.out.println(">>>线程" + this.getId() + "返回:" + info);
                        if (tempStr!=null) {
                            outputStream.write(tempStr.getBytes());
                        }
                        outputStream.flush();
                        //重置,不然每次收到的数据都会累加起来
                        info = "";
                       // System.out.println(">>>线程" + this.getId() + "回应:" + responseStr);
                    }
                }
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        //关闭资源
        finally
        {
            System.out.println(">>>线程" + this.getId() + "的连接已断开\n");
            try
            {
                if (outputStream != null)
                    outputStream.close();
                if (inputStream != null)
                    inputStream.close();
                if (m_socket != null)
                    m_socket.close();
                m_isRuning = false;
            }
            catch (IOException e)
            {
                e.printStackTrace();
            }
        }
    }





}
